#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.utils

Module providing utility functions for scikit-chem
"""

from .suppress import Suppressor
