#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.io

Module defining input and output methods in scikit-chem.

"""

from .sdf import read_sdf
from .smiles import read_smiles
