#! /usr/bin/env python
#
# Copyright (C) 2007-2009 Rich Lewis <rl403@cam.ac.uk>
# License: 3-clause BSD

"""
skchem.target_prediction

Module implementing target prediction algorithms.

"""

from .target_prediction import AbstractTargetPredictionAlgorithm
from .PIDGIN import PIDGIN
